//-------------------------------------------------------------------------------------------------+
// GlobalMapManager.cpp
// n}ǗRTC
//-------------------------------------------------------------------------------------------------+

#include "GlobalMapManager.h"
#include "GlobalMapFileStream.h"
#include <iostream>

using namespace std;


//-------------------------------------------------------------------------------------------------+
// Module specification
//-------------------------------------------------------------------------------------------------+
static const char* globalmapmanager_spec[] =
  {
    "implementation_id", "GlobalMapManager",
    "type_name",         "GlobalMapManager",
    "description",       "manage and output global map and area info data",
    "version",           "1.0.0",
    "vendor",            "aisl.cs.tut.ac.jp",
    "category",          "GlobalMa",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.filename", "areainfo.dat",
    // Widget
    "conf.__widget__.filename", "text",
    // Constraints
    ""
  };


//-------------------------------------------------------------------------------------------------+
// * @brief constructor
//-------------------------------------------------------------------------------------------------+
GlobalMapManager::GlobalMapManager(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_personsIn("persons", m_persons),
    m_AbsoluteMapServicePort("AbsoluteMapService"),
    m_AbsoluteMapService( &mapdata, &cs_mapdata ),
    m_AreaInfoServicePort("AreaInfoService"),
    m_AreaInfoService( &areainfo, &cs_areainfo )
{

	cerr << "------------------------------------------------------------+\n"
		 << " GlobalMapManager Component (Ver. 8th February 2012)\n"
		 << "------------------------------------------------------------+\n\n";

	InitializeCriticalSection( &cs_mapdata );
	InitializeCriticalSection( &cs_areainfo );
}


//-------------------------------------------------------------------------------------------------+
// * @brief destructor
//-------------------------------------------------------------------------------------------------+
GlobalMapManager::~GlobalMapManager(){
}


//-------------------------------------------------------------------------------------------------+
// 
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapManager::onInitialize(){

	// Set InPort buffers
	addInPort("persons", m_personsIn);
  
	// Set service provider to Ports
	m_AbsoluteMapServicePort.registerProvider("AbsoluteMapService", "MRFC::AbsoluteMapService", m_AbsoluteMapService);
	m_AreaInfoServicePort.registerProvider("AreaInfoService", "TUT::AreaInfoService", m_AreaInfoService);

	// Set CORBA Service Ports
	addPort(m_AbsoluteMapServicePort);
	addPort(m_AreaInfoServicePort);

	// Bind variables and configuration variable
	bindParameter("filename", m_filename, "areainfo.dat");

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// ̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapManager::onActivated(RTC::UniqueId ec_id){

	EnterCriticalSection( &cs_areainfo );
	EnterCriticalSection( &cs_mapdata );
	
	GlobalMapFileInStream load( m_filename.c_str() );

	mapdata  = *( load.GetOctMapData() );
	areainfo = *( load.GetAreaInfo() );

	LeaveCriticalSection( &cs_mapdata );
	LeaveCriticalSection( &cs_areainfo );

	subareas = ConvertSubAreas( &areainfo );

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// 񊈐̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapManager::onDeactivated(RTC::UniqueId ec_id){

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// s郁CvZXI
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapManager::onExecute(RTC::UniqueId ec_id){

	if( m_personsIn.isNew() ){

		while( !m_personsIn.isEmpty() ) m_personsIn.read();

		CalcPopulation();
		SetPopulation();
	}

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// AreaInfoTuGAɕϊ
//-------------------------------------------------------------------------------------------------+
SubAreas GlobalMapManager::ConvertSubAreas( TUT::TimedAreaInfo *areainfo ){

	SubAreas subareas;

	EnterCriticalSection( &cs_areainfo );

	for( int i=0; i< areainfo->data.length(); i++ ){

		Vertexes vert;

		for( int v=0; v<areainfo->data[i].vertexes.length(); v++ ){

			vert.push_back( Coordinate( areainfo->data[i].vertexes[v].x, areainfo->data[i].vertexes[v].y ) );
		}

		SubArea area( areainfo->data[i].id, vert );

		area.population = areainfo->data[i].population;

		for( int l=0; l<areainfo->data[i].links.length(); l++ ){

			Coordinate c[2];

			c[0].x = areainfo->data[i].links[l].boder.p1.x;
			c[0].y = areainfo->data[i].links[l].boder.p1.y;
			c[1].x = areainfo->data[i].links[l].boder.p2.x;
			c[1].y = areainfo->data[i].links[l].boder.p2.y;

			area.AddLink( areainfo->data[i].links[l].id, Line( c[0], c[1] ) );
		}

		subareas.push_back( area );
	}

	LeaveCriticalSection( &cs_areainfo );

	return subareas;
}


//-------------------------------------------------------------------------------------------------+
// AreaInfo̐lZbg
//-------------------------------------------------------------------------------------------------+
void GlobalMapManager::ResetPopulation(){

	EnterCriticalSection( &cs_areainfo );

	for( int i=0; i<(int)areainfo.data.length(); i++ ){

		areainfo.data[i].population = 0;
	}

	LeaveCriticalSection( &cs_areainfo );
}


//-------------------------------------------------------------------------------------------------+
// AreaInfoɃTuGAlZbg
// TuGAAreaInfȏΉĂ邱Ƃ
//-------------------------------------------------------------------------------------------------+
void GlobalMapManager::SetPopulation(){

	if( areainfo.data.length() != subareas.size() ) return;

	EnterCriticalSection( &cs_areainfo );

	for( int i=0; i<(int)areainfo.data.length(); i++ ){

		areainfo.data[i].population = subareas[i].population;
	}

	LeaveCriticalSection( &cs_areainfo );
}


//-------------------------------------------------------------------------------------------------+
// eTuGA̐lvZ
//-------------------------------------------------------------------------------------------------+
void GlobalMapManager::CalcPopulation(){

	// ܂̓Zbg
	for( int a=0; a<(int)subareas.size(); a++ ) subareas[a].population = 0;

	for( int i=0; i<(int)m_persons.data.length(); i++ ){

		for( int a=0; a<(int)subareas.size(); a++ ){

			if( subareas[a].InSide( m_persons.data[i].x, m_persons.data[i].y ) ){

				subareas[a].population++;
				break;
			}
		}
	}
}



//-------------------------------------------------------------------------------------------------+
extern "C"
{
  void GlobalMapManagerInit(RTC::Manager* manager)
  {
    coil::Properties profile(globalmapmanager_spec);
    manager->registerFactory(profile,
                             RTC::Create<GlobalMapManager>,
                             RTC::Delete<GlobalMapManager>);
  }
};
