//-------------------------------------------------------------------------------------------------+
// MRFCSVC_impl.cpp
// Service implementation code of MRFC.idl
//-------------------------------------------------------------------------------------------------+

#include "MRFCSVC_impl.h"


//-------------------------------------------------------------------------------------------------+
// RXgN^
//-------------------------------------------------------------------------------------------------+
AbsoluteMapServiceSVC_impl::AbsoluteMapServiceSVC_impl( MRFC::TimedAbsoluteOGMapData *globalmap, CRITICAL_SECTION *cs ){

	mapdata   = globalmap;
	c_section = cs;
}


//-------------------------------------------------------------------------------------------------+
// fXgN^
//-------------------------------------------------------------------------------------------------+
AbsoluteMapServiceSVC_impl::~AbsoluteMapServiceSVC_impl(){
}


//-------------------------------------------------------------------------------------------------+
// ݒn
//-------------------------------------------------------------------------------------------------+
RTC::OGMapConfig AbsoluteMapServiceSVC_impl::getAbsoluteOGMapConfig(){

	return mapdata->mapconfig;
}


//-------------------------------------------------------------------------------------------------+
// n}n
//-------------------------------------------------------------------------------------------------+
MRFC::TimedAbsoluteOGMapData *AbsoluteMapServiceSVC_impl::getAbsoluteOGMap(CORBA::Double x, CORBA::Double y, CORBA::ULong width, CORBA::ULong height){

	EnterCriticalSection( c_section );

	MRFC::TimedAbsoluteOGMapData *globalmap = new MRFC::TimedAbsoluteOGMapData();

	globalmap->mapconfig = mapdata->mapconfig;
	globalmap->tm        = mapdata->tm;

	globalmap->mapconfig.origin.position.x = x;
	globalmap->mapconfig.origin.position.y = y;

	globalmap->cells.length( width * height );

	int index = 0;

	for( int j=0; j<height; j++ ) for( int i=0; i<width; i++ ){

		int cell_x = i + (int)(( x - mapdata->mapconfig.origin.position.x ) / globalmap->mapconfig.xScale );
		int cell_y = j + (int)(( y - mapdata->mapconfig.origin.position.y ) / globalmap->mapconfig.yScale );

		// ͈͊O͖ϑƂ
		if( cell_x < 0 || cell_y < 0 || globalmap->mapconfig.width <= cell_x || globalmap->mapconfig.height <= cell_y ){

			globalmap->cells[index] = -1;
		}
		else globalmap->cells[index] = mapdata->cells[cell_y * mapdata->mapconfig.width + cell_x];

		index++;
	}

	LeaveCriticalSection( c_section );

	return globalmap;
}


//-------------------------------------------------------------------------------------------------+
// n}n
//-------------------------------------------------------------------------------------------------+
MRFC::TimedFloatAbsoluteOGMapData *AbsoluteMapServiceSVC_impl::getFloatAbsoluteOGMap(CORBA::Double x, CORBA::Double y, CORBA::ULong width, CORBA::ULong height){

	EnterCriticalSection( c_section );

	MRFC::TimedFloatAbsoluteOGMapData *globalmap = new MRFC::TimedFloatAbsoluteOGMapData();

	globalmap->mapconfig = mapdata->mapconfig;
	globalmap->tm        = mapdata->tm;

	globalmap->mapconfig.origin.position.x = x;
	globalmap->mapconfig.origin.position.y = y;

	globalmap->cells.length( width * height );

	int index = 0;

	for( int j=0; j<height; j++ ) for( int i=0; i<width; i++ ){

		int cell_x = i + (int)(( x - mapdata->mapconfig.origin.position.x ) / globalmap->mapconfig.xScale );
		int cell_y = j + (int)(( y - mapdata->mapconfig.origin.position.y ) / globalmap->mapconfig.yScale );

		// ͈͊O͖ϑƂ
		if( cell_x < 0 || cell_y < 0 || globalmap->mapconfig.width <= cell_x || globalmap->mapconfig.height <= cell_y ){

			globalmap->cells[index] = -1;
		}
		else globalmap->cells[index] = (float)mapdata->cells[cell_y * mapdata->mapconfig.width + cell_x] * 0.01;

		index++;
	}

	LeaveCriticalSection( c_section );

	return globalmap;
}
